@@ -57,14 +57,23 @@ Paul was named <head item="art" instance="art.30002">Art</head> magazine's top c
 
 If the sense information is not there, then you can not use the --key FILE option
 
+We also added a candidate tag so if would like to specifiy the possible senses 
+for the target word in the instance, you can include a candidate attribute in 
+which the candidate senses are seperated by a comma. For example: 
+
+<head item="target word" instance="id" sense="sense" candidate="s1,s2">word</head>.
+
+You must use the --candidate option to actually use the candidate information, 
+otherwise it will be ignored.
+
 =head3 --sval2
 
 The format is in sval2 format
 
-=head3 --metamap
+=head3 --mmxml
 
-The format is in metamap xml format in which each target word though
-is identified by a <Target></Target> tag similar to that of the 
+The format is in metamap xml (mmxml) format in which each target word 
+thoughis identified by a <Target></Target> tag similar to that of the 
 <Token></Token> tags. 
 
 We have a conversion program in the coverters/ directory which will 
@@ -74,7 +83,7 @@ called: plain2mm-xml.pl
 =head3 --candidates
 
 This option uses the candidate senses as identified by metamap for 
-the target word. This option can only be used with the --metamap option. 
+the target word. This option can only be used with the --mmxml option. 
 
 =head3 --senses DIR|File
 
@@ -292,22 +301,20 @@ See create-icpropagation.pl for more information.
 =head3 --vectormatrix FILE
 
 This is the matrix file that contains the vector information to 
-use with the vector measure. This is required if you specify vector 
-with the --measure option. 
+use with the vector measure. 
 
-This file is generated by the vector-input.pl program. An example 
-of this file can be found in the samples/ directory and is called 
-matrix.
+If you do not want to use the default, this file is generated by 
+the vector-input.pl program. An example of this file can be found 
+in the samples/ directory and is called matrix.
 
 =head3 --vectorindex FILE
 
 This is the index file that contains the vector information to 
-use with the vector measure. This is required if you specify vector 
-with the --measure option.
+use with the vector measure. 
 
-This file is generated by the vector-input.pl program. An example 
-of this file can be found in the samples/ directory and is called 
-index.
+If you do not want to use the default, this file is generated by 
+the vector-input.pl program. An example of this file can be found 
+in the samples/ directory and is called index.
 
 
 =head3 --debugfile FILE
@@ -465,7 +472,7 @@ use XML::Twig;
 use File::Spec;
 
 
-eval(GetOptions( "version", "help", "username=s", "password=s", "hostname=s", "database=s", "socket=s", "measure=s", "config=s", "forcerun", "debug", "icpropagation=s", "realtime", "stoplist=s", "vectorstoplist=s", "leskstoplist=s", "vectormatrix=s", "vectorindex=s", "defraw", "dictfile=s", "t", "stem", "window=s", "key", "log=s", "senses=s", "plain", "sval2", "metamap", "candidates", "compound", "trace=s", "undirected")) or die ("Please check the above mentioned option(s).\n");
+eval(GetOptions( "version", "help", "username=s", "password=s", "hostname=s", "database=s", "socket=s", "measure=s", "config=s", "forcerun", "debug", "icpropagation=s", "realtime", "stoplist=s", "vectorstoplist=s", "leskstoplist=s", "vectormatrix=s", "vectorindex=s", "defraw", "dictfile=s", "t", "stem", "window=s", "key", "log=s", "senses=s", "plain", "sval2", "mmxml", "candidates", "compound", "trace=s", "undirected", "st")) or die ("Please check the above mentioned option(s).\n");
 
 
 my $debug = 0;
@@ -494,13 +501,14 @@ if(scalar(@ARGV) < 1) {
 my $source = shift;
 
 #  initialize variables
-my $precision    = "";
-my $floatformat  = "";
-my $measure      = "";
-my $window       = "";
-my %sensehash    = ();
-my %instancehash = ();
-my %keyhash      = ();
+my $precision     = "";
+my $floatformat   = "";
+my $measure       = "";
+my $window        = "";
+my %sensehash     = ();
+my %candidatehash = ();
+my %instancehash  = ();
+my %keyhash       = ();
 
 #  check and set the options
 &checkOptions       ();
@@ -553,8 +561,9 @@ sub setInput() {
     #  load the input into the instance hash
     foreach my $inputfile (@inputfiles) { 
 
-	if(defined $opt_sval2) { &load_Sval2_Input($inputfile); }
-	else                   { &load_Plain_Input($inputfile); }
+	if(defined $opt_sval2)    { &load_Sval2_Input($inputfile);   }
+	elsif(defined $opt_mmxml) { &load_MetaMap_Input($inputfile); }
+	else                      { &load_Plain_Input($inputfile);   }
     }
 }
 
@@ -566,6 +575,8 @@ sub assignSenses {
     
     foreach my $tw (sort keys %instancehash) { 
 	
+	if($tw=~/^\s*$/) { next; }
+
 	if($debug) { print STDERR "  Assigning sense for $tw\n"; }
 
 	#  open the log files
@@ -574,11 +585,15 @@ sub assignSenses {
 	#  get the possible senses for the target words
 	my @senses = (); 
 	
-	if(defined $opt_senses) {
+	#  if defined --senese similar to what we do with the NLM-WSD datset
+	#  where the target word has predefined set of senses
+	if(defined $opt_senses) { 
 	    foreach my $sense (sort keys %{$sensehash{$tw}}) { 
-		push @senses, $sense; }
+		push @senses, $sense; 
+	    }
 	}
 	
+	
 	#  loop through the instances and assign each of them a sense
 	my $tp = 0; my $tn = 0;
 	foreach my $id (sort keys %{$instancehash{$tw}}) {
@@ -586,13 +601,27 @@ sub assignSenses {
 	    my $instance = $instancehash{$tw}{$id};
 	    
 	    my $hashref = undef;
+	    
+	    #  if the --senses option was defined pass the senses given 
+	    #  for the specified target word
 	    if(defined $opt_senses) {
 		($hashref) = $senserelate->assignSense($tw, $instance, \@senses);
 	    }
+	    #  if the --candidates option was defined pass the senses given 
+	    #  for the instance containing the target word
+	    elsif(defined $opt_candidates) { 
+		my @candidates = ();
+		foreach my $sense (sort keys %{$candidatehash{$tw}{$id}}) { 
+		    push @candidates, $sense; 
+		}
+	       
+		($hashref) = $senserelate->assignSense($tw, $instance, \@candidates);
+	    }
+	    #  no option was defined so the candidates are coming from the UMLS
 	    else { 
 		($hashref) = $senserelate->assignSense($tw, $instance, undef);
 	    }
-	    
+
 	    if(defined $hashref) {
 		
 		foreach my $sense (sort keys %{$hashref}) { 
@@ -601,6 +630,10 @@ sub assignSenses {
 		}
 
 	    }
+	    else {
+		print STDERR "ERROR: A sense was not assigned to instance to $tw for $id\n";
+		exit;
+	    }
 	}
 	close ANSWERS;
     }
@@ -709,7 +742,7 @@ sub load_MetaMap_Input {
 
     my $inputfile = shift;
     
-    if($debug) { print STDERR "In loadPlainInput for $inputfile\n"; }
+    if($debug) { print STDERR "In load_MetaMap_Input for $inputfile\n"; }
     
     #  open the input file
     open(INPUT, $inputfile) || die "Could not open input file: $inputfile\n";
@@ -722,12 +755,33 @@ sub load_MetaMap_Input {
 	}
 	$abstract .= $_;
     }
+    
+    my $abstractid = 0; 
+    if($#abstracts < 0) { 
+	print STDERR "\nERROR. This file does not seem to be in the proper format.\n";
+	print STDERR "Please check the format of your input file.\n\n";
+	&minimalUsageNotes();
+	exit;
+    }
 
-    my $abstractid = 0;
     foreach my $abstract (@abstracts) { 
 	
 	if($abstract=~/^\s*$/) { next; }
 	
+	if($abstract=~/<instance id>/) { 
+	    print STDERR "\nERROR. This file looks like it is in sval2 format.\n";
+	    print STDERR "Please check the format of your input file.\n\n";
+	    &minimalUsageNotes();
+	    exit;
+	}
+	if($abstract=~/<head item=>/) { 
+	    print STDERR "\nERROR. This file looks like it is in plain format.\n";
+	    print STDERR "Please check the format of your input file.\n\n";
+	    &minimalUsageNotes();
+	    exit;
+	}
+	
+
 	# increment id
 	$abstractid++;
 	
@@ -736,42 +790,72 @@ sub load_MetaMap_Input {
 	print OUTFILE "<text id=\"$aid\">\n";
 	
 	#  set the xml file for this abstract
-	system "rm $infile.processing";    
-	open(FILE, ">$infile.processing") || die "Could not open $infile.processing\n";
-	print FILE "$abstract";
-	close FILE;
+	if(-e "$inputfile.processing") { system "rm $inputfile.processing"; }
+	open(FILE, ">$inputfile.processing") || 
+	    die "Could not open $infile.processing\n";
+	print FILE "$abstract";	close FILE;
 	
 	#  load the metamap xml output
 	my $t= XML::Twig->new();
-	$t->parsefile("$infile.processing");
+	$t->parsefile("$inputfile.processing");
 	my $root = $t->root;
 	
+	#  initialize the instance variables
+	my $id = ""; my $tw = ""; my $item = ""; my $sense = ""; 
+	my $method= $root; my $tokenid = 0; my $tflag = 0;
+	my @tokens = (); my @cuis = (); my @matches = (); 
+	my %candidates = ();
+
 	#  loop through tokens
-	my $method= $root; my $id = 0; my $instance = "";
 	while( $method=$method->next_elt( $root )) { 
+
+	    if($method->local_name eq "UttText") { 
+		$sentenceid++; 
+		$tokenid = 0;
+	    }
 	    
-	    if($method->local_name eq "Utterances") { 
-		if($id != 0) { 
-		    $instancehash{$tw}{$id} = $instance;
+	    #  get the tokens
+	    if($method->local_name eq "InputMatch") { 
+		my $token = $method->text; push @tokens, $token;
+	    }
+	    
+	    #  check if target word
+	    if($method->local_name eq "Target") { 
+		
+		#  get the attributes
+		$id=$method->att('instance');
+		$tw=$method->att('item');
+		$sense=$method->att('sense');
+		
+		#  remove the target word
+		my $target = pop @tokens;
+		
+		#  set the target word with its attributes
+		$item = "<head ";
+		$item .= "item=\"$tw\" ";
+		$item .= "instance=\"$id\" ";
+		$item .= "sense=\"$sense\"";
+		$item .= ">$target<\/head>";
+
+		#  add the updated target word
+		push @tokens, $item;
+		
+		#  if the key is defined add sense to the key hash
+		if(defined $opt_key) { 
+		    $keyhash{$tw}{$id} = "$tw%$sense";
 		}
-		$id++; 
-		$instance = "";
 	    }
 	    
-	    if($method->local_name eq "Token") { 
-		my $token = $method->text; $instance .= "$token ";
-	    }
-	
 	    #  check if in mapping
 	    if($method->local_name eq "Mapping") { $flag = 1; }
 	    
 	    #  if in mapping, get the cui
-	    if( ($method->local_name eq "CandidateCUI") && ($flag = 1) ) { 
-	    my $cui = $method->text; push @cuis, $cui;
+	    if( ($method->local_name eq "CandidateCUI") && ($flag == 1) ) { 
+		my $cui = $method->text; push @cuis, $cui;
 	    }
-	    
+
 	    #  if in mapping, get the cui
-	    if( ($method->local_name eq "CandidateMatched") && ($flag = 1) ) { 
+	    if( ($method->local_name eq "CandidateMatched") && ($flag == 1) ) { 
 		my $match = $method->text; 
 		$match=~s/[\*\?\+\(\)\[\]\/ ]//g; 
 		push @matches, lc($match);
@@ -784,43 +868,40 @@ sub load_MetaMap_Input {
 		    my $term = $matches[$i]; my $mflag = 0;
 		    while($mflag == 0) {
 			foreach my $token (@tokens) {
-			$token=lc($token);
-			print "$token : $term ";
-			if($token=~/$term/) {
-			    $mappings{$token}{"$cuis[$i]/$matches[$i]"}++;
-			    $mflag = 1; print "mapping";
+			    my $word = $token;
+			    if($token=~/<head (.*?)>(.*?)<\/head>/) {
+				$word = $2;
+			    } 
+			    else { $token=lc($token); }
+			    if($word=~/$term/) {
+				if($token=~/<head/) { 
+				    $candidates{$cuis[$i]}++;
+				}
+				$mappings{$token}{"$cuis[$i]/$matches[$i]"}++;
+				$mflag = 1; 
+			    }
 			}
-			print "\n";
-			}
-			print "====================================\n";
 			chop $term;
 			if($term=~/^\s*$/) { $mflag = 1; }
 		    }
 		}
-		foreach my $token (@tokens) { 
-		    my $tok = lc($token);
-		    $tokenid++;
-		    
-		    my $a = sprintf("%03d", $abstractid);
-		    my $s = sprintf("%03d", $sentenceid);
-		    my $t = sprintf("%03d", $tokenid);
-		    my $id = "d$a.s$s.t$t";
-		    
-		    my $senses = "";
-		    foreach my $m (sort keys %{$mappings{$tok}}) { 
-			$m=~/(C[0-9]+)\//;
-			$senses .= "$1,";
-		    } chop $senses;
-
-		    if($senses=~/^\s*$/) { print OUTFILE "$token\n"; }
-		    else                 { print OUTFILE "<head id=\"$id\" candidates=\"$senses\">$token<\/head>\n"; }
-		}
-		@tokens  = ();;
-		@cuis    = ();
-		@matches = ();
 	    }
+	    
 	}
 
+	#  store the candidates in the candidate hash if defined
+	if(defined $opt_candidates) { 
+	    my @senses = ();
+	    foreach my $cui (sort keys %candidates) { 
+		$candidatehash{$tw}{$id}{$cui}++;
+	    }
+	}
+	
+	#  store instance in the hash
+	$instancehash{$tw}{$id} = join " ", @tokens;
+	
+	#  remove the processing file
+	system "rm $inputfile.processing"; 
     }
 }
 
@@ -846,14 +927,35 @@ sub load_Plain_Input {
 	    exit;
 	}
 	    
-	my $before = "";	my $tw     = "";	
-	my $id     = "";        my $sense  = "";	
-	my $after  = "";
+	my $before = "";	my $tw        = "";	
+	my $id     = "";        my $sense     = "";	
+	my $after  = "";        my $candidate = "";
 	
-	if($_=~/^(.*?)<head item=\"(.*?)\" instance=\"(.*?)\" sense=\"(.*?)\">(.*?)<\/head>(.*?)$/) { 
+	
+	if($_=~/^(.*?)<head item=\"(.*?)\" instance=\"(.*?)\" sense=\"(.*?)\" candidates=\"(.*?)\">(.*?)<\/head>(.*?)$/) { 
+	    $before    = $1;	    $tw     = $2;
+	    $id        = $3;	    $sense  = $4;
+	    $candidate = $5;        $after  = $7;
+	    if( (defined $opt_candidates) && ($candidate=~/^\s*$/) ) { 
+		print STDERR "\nERROR: The candidate information is missing from instance.\n";
+		print STDERR "$_\n\n";
+		&minimalUsageNotes();
+		exit;
+	    }
+
+	}
+	elsif($_=~/^(.*?)<head item=\"(.*?)\" instance=\"(.*?)\" sense=\"(.*?)\">(.*?)<\/head>(.*?)$/) { 
 	    $before = $1;	    $tw     = $2;
 	    $id     = $3;	    $sense  = $4;
 	    $after  = $6;
+
+	    if( (defined $opt_candidates) ) { 
+		print STDERR "\nERROR: The candidate information is missing from instance.\n";
+		print STDERR "$_\n\n";
+		&minimalUsageNotes();
+		exit;
+	    }
+
 	}
 	elsif($_=~/^(.*?)<head item=\"(.*?)\" instance=\"(.*?)\">(.*?)<\/head>(.*?)$/) { 
 	    $before = $1;	    $tw     = $2;
@@ -879,15 +981,30 @@ sub load_Plain_Input {
 	    &minimalUsageNotes();
 	    exit;
 	}
-	    	    
-	#  reset the instance with out the sense information if it is there
+
+	#  check that the candidate information is there
+	if( (defined $opt_candidates) && ($candidate=~/^\s*$/) ) { 
+	    print STDERR "\nERROR: The candidate information is missing from instance.\n";
+	    print STDERR "$_\n\n";
+	    &minimalUsageNotes();
+	    exit;
+	}
+
+	#  reset the instance with out the sense or candidate information if it is there
 	my $instance = $before . " <head item=\"$tw\" instance=\"$id\">$tw<\/head> " . $after;
 
-	#  send the instance to 
+	#  remove non essential white space
 	$instance=~s/\s+/ /g;	$instance=~s/^\s*//g;	$instance=~s/\s*$//g;
 	
+	#  store the insance
 	$instancehash{$tw}{$id} = $instance;
-	
+	 
+	#  store the candidate infromtion 
+	if(defined $opt_candidates) { 
+	    my @array = split/\,/, $candidate;
+	    foreach my $cui (@array) { $candidatehash{$tw}{$id}{$cui}++; }
+	}
+
 	#  if the --key option is defined print out the key information
 	if(defined $opt_key) { 
 	    if($sense eq "")  { 
@@ -1017,21 +1134,35 @@ sub load_UMLS_Similarity {
 	use UMLS::Similarity::nam;
 	$meas = UMLS::Similarity::nam->new($umls);
     }
+
+    my %ic_hash = ();
+    if(defined $opt_icpropagation) {
+	$ic_hash{"icpropagation"} = $opt_icpropagation;
+    }
+    if(defined $opt_icfrequency) { 
+	$is_hash{"icfrequency"} = $opt_icfrequency;
+    }
+
+
     #  load the module implementing the Resnik (1995) measure
     if($measure eq "res") {
+
+	if(defined $opt_st) { 
+	    $ic_hash{"st"} = 1;
+	}
 	use UMLS::Similarity::res;
-	$meas = UMLS::Similarity::res->new($umls);
+	$meas = UMLS::Similarity::res->new($umls, \%ic_hash);
     }
     #  load the module implementing the Jiang and Conrath 
     #  (1997) measure
     if($measure eq "jcn") {
 	use UMLS::Similarity::jcn;
-	$meas = UMLS::Similarity::jcn->new($umls);
+	$meas = UMLS::Similarity::jcn->new($umls, \%ic_hash);
     }
     #  load the module implementing the Lin (1998) measure
     if($measure eq "lin") {
 	use UMLS::Similarity::lin;
-	$meas = UMLS::Similarity::lin->new($umls);
+	$meas = UMLS::Similarity::lin->new($umls, \%ic_hash );
     }
     #  load the module implementing the random measure
     if($measure eq "random") {
@@ -1102,12 +1233,6 @@ sub load_UMLS_Interface {
     if(defined $opt_undirected) { 
 	$options_hash{"realtime"} = $opt_undirected;
     }
-    if(defined $opt_icpropagation) {
-	$option_hash{"icpropagation"} = $opt_icpropagation;
-    }
-    if(defined $opt_icfrequency) { 
-	$option_hash{"icfrequency"} = $opt_icfrequency;
-    }
     if(defined $opt_smooth) { 
 	$option_hash{"smooth"} = $opt_smooth;
     }
@@ -1131,6 +1256,12 @@ sub checkOptions {
 
     if($debug) { print STDERR "In checkOptions\n"; }
 
+    if((defined $opt_candidates) && (defined $opt_sval2)) { 
+	print STDERR "The --candidates option is not available for the sval2 format.\n";
+	&minimalUsageNotes();
+	exit;
+    }
+	
     if(defined $opt_measure) {
 	if($opt_measure=~/\b(path|wup|lch|cdist|nam|vector|res|lin|random|jcn|lesk)\b/) {
 	    #  good to go
@@ -1143,27 +1274,6 @@ sub checkOptions {
 	}   
     }
     
-    # make certain the db file is specified if the vector measure 
-    # is being used
-    if($opt_measure=~/vector/) {
-	if(! (defined $opt_vectorindex)) {
-	    print STDERR "The --vectorindex and --vectormatrix option must be\n";
-	    print STDERR "specified when using the vector measure. An example\n";
-	    print STDERR "of the matrix and index files can be seen in the \n";
-	    print STDERR "samples/ directory.\n\n";
-	    &minimalUsageNotes();
-	    exit;
-	}
-	if(! (defined $opt_vectormatrix)) {
-	    print STDERR "The --vectorindex and --vectormatrix option must be\n";
-	    print STDERR "specified when using the vector measure.An example\n";
-	    print STDERR "of the index and matrix files can be seen in the\n";
-	    print STDERR "samples/ directory. The vector-input.pl program can\n";
-	    print STDERR "generate these files given your specific text. \n\n";
-	    &minimalUsageNotes();
-	    exit;
-	}
-    }
      if(defined $opt_stem) { 
 	if(! ($opt_measure=~/vector|lesk/) ) {
 	    print STDERR "The --stem option is only available\n";
@@ -1218,19 +1328,6 @@ sub checkOptions {
 	}
     }    
     
-    #  make certain the propagation file is specified if the resnik, 
-    #  jcn, or lin measure is being used
-    if($opt_measure=~/(res|lin|jcn)/) {
-	if(! (defined $opt_icpropagation) and !(defined $opt_icfrequency) ) {
-	    print STDERR "The --icpropagation or --icfrequency option must be\n";
-	    print STDERR "specified when using the res, lin or jcn measures.\n";
-	    print STDERR "An example of the propagation file can be seen in\n";
-	    print STDERR "the samples/ directory.\n\n";
-	    &minimalUsageNotes();
-	    exit;
-	}
-    } 
-
     #  the --smooth option can only be used with the icfrequency options
     if(defined $opt_smooth) {
 	if(!defined $opt_icfrequency) {
@@ -1316,8 +1413,9 @@ sub setOptions {
     $floatformat = join '', '%', '.', $precision, 'f';
     
     #  UMLS::SenseRelate Options
-    if(defined $opt_senses)   { $set .= "  --senses $opt_senses\n";     }    
-    if(defined $opt_stoplist) { $set .= "  --stoplist $opt_stoplist\n"; }
+    if(defined $opt_senses)     { $set .= "  --senses $opt_senses\n";     }    
+    if(defined $opt_stoplist)   { $set .= "  --stoplist $opt_stoplist\n"; }
+    if(defined $opt_candidates) { $set .= "  --candidates\n";             }
 
     if(defined $opt_window)   { 
 	$window = $opt_window;
@@ -1432,6 +1530,10 @@ sub setOptions {
 	$set .= "  --stem\n";
     }
 
+    if(defined $opt_st) { 
+	$set .= "  --st\n";
+    }
+
     #  print settings
     if($default ne "") { 
 	print STDERR "Default Options:\n";
@@ -1485,6 +1587,8 @@ sub showHelp() {
     
     print "--sval2                  INPUTFILE is in sval2 format.\n\n";
     
+    print "--mmxml                  INPUTFILE is in mmxml format.\n\n";
+
     print "--senses FILE|DIR        File or directory containing the sense files\n\n";
 
     print "--measure MEASURE        The measure to use to calculate the\n";
@@ -1540,10 +1644,10 @@ sub showHelp() {
 
     print "\n\nVector Options:\n\n";
 
-    print "--vectormatrix FILE        The matrix file containing the vector\n";
+    print "--vectormatrix FILE      The matrix file containing the vector\n";
     print "                         information for the vector measure.\n\n";
 
-    print "--vectorindex FILE         The index file containing the vector\n";
+    print "--vectorindex FILE       The index file containing the vector\n";
     print "                         information for the vector measure.\n\n";
     
 
@@ -1576,7 +1680,7 @@ sub showHelp() {
 #  function to output the version number
 ##############################################################################
 sub showVersion {
-    print '$Id: umls-targetword-senserelate.pl,v 1.3 2011/04/18 16:55:39 btmcinnes Exp $';
+    print '$Id: umls-targetword-senserelate.pl,v 1.9 2011/05/20 19:09:12 btmcinnes Exp $';
     print "\nCopyright (c) 2010-2011, Ted Pedersen & Bridget McInnes\n";
 }
 
